@@ -185,6 +185,10 @@ DownloadImpl::setLastError(const QDBusError& err) {
 
 void
 DownloadImpl::start() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} start())").arg(_id));
     QDBusPendingCall call =
         _dbusInterface->start();
@@ -195,6 +199,10 @@ DownloadImpl::start() {
 
 void
 DownloadImpl::pause() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} pause())").arg(_id));
     QDBusPendingCall call =
         _dbusInterface->pause();
@@ -205,6 +213,10 @@ DownloadImpl::pause() {
 
 void
 DownloadImpl::resume() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} resume())").arg(_id));
     QDBusPendingCall call =
         _dbusInterface->resume();
@@ -215,6 +227,10 @@ DownloadImpl::resume() {
 
 void
 DownloadImpl::cancel() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} cancel())").arg(_id));
     QDBusPendingCall call =
         _dbusInterface->cancel();
@@ -225,6 +241,10 @@ DownloadImpl::cancel() {
 
 void
 DownloadImpl::collected() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} collected()").arg(_id));
     QDBusPendingReply<> reply =
         _dbusInterface->collected();
@@ -238,6 +258,10 @@ DownloadImpl::collected() {
 
 void
 DownloadImpl::allowMobileDownload(bool allowed) {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug,
         QString("Download{%1} allowMobileDownload%2())").arg(_id).arg(allowed));
     QDBusPendingReply<> reply =
@@ -252,6 +276,10 @@ DownloadImpl::allowMobileDownload(bool allowed) {
 
 bool
 DownloadImpl::isMobileDownloadAllowed() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return false;
+    }
     Logger::log(Logger::Debug,
         QString("Download{%1} isMobileDownloadAllowed").arg(_id));
     QDBusPendingReply<bool> reply =
@@ -270,6 +298,10 @@ DownloadImpl::isMobileDownloadAllowed() {
 
 void
 DownloadImpl::setDestinationDir(const QString& path) {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug, QString("Dowmload{%1} setDestinationDir(%2)")
         .arg(_id).arg(path));
     QDBusPendingReply<> reply =
@@ -284,6 +316,10 @@ DownloadImpl::setDestinationDir(const QString& path) {
 
 void
 DownloadImpl::setHeaders(QMap<QString, QString> headers) {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug,
         QString("Download {%1} setHeaders(%2)").arg(_id), headers);
 
@@ -299,6 +335,11 @@ DownloadImpl::setHeaders(QMap<QString, QString> headers) {
 
 QVariantMap
 DownloadImpl::metadata() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        QVariantMap emptyResult;
+        return emptyResult;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} metadata()").arg(_id));
     QDBusPendingReply<QVariantMap> reply =
         _dbusInterface->metadata();
@@ -317,6 +358,10 @@ DownloadImpl::metadata() {
 
 void
 DownloadImpl::setMetadata(QVariantMap map) {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug,
         QString("Download {%1} setMetadata(%2)").arg(_id), map);
 
@@ -332,6 +377,11 @@ DownloadImpl::setMetadata(QVariantMap map) {
 
 QMap<QString, QString>
 DownloadImpl::headers() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        QMap<QString, QString> empty;
+        return empty;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} headers()").arg(_id));
     QDBusPendingReply<QMap<QString, QString> > reply =
         _dbusInterface->headers();
@@ -351,6 +401,10 @@ DownloadImpl::headers() {
 
 void
 DownloadImpl::setThrottle(qulonglong speed) {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return;
+    }
     Logger::log(Logger::Debug,
         QString("Download{%1} setThrottle(%2)").arg(_id).arg(speed));
     QDBusPendingReply<> reply =
@@ -365,6 +419,10 @@ DownloadImpl::setThrottle(qulonglong speed) {
 
 qulonglong
 DownloadImpl::throttle() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return 0;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} throttle()").arg(_id));
     QDBusPendingReply<qulonglong> reply =
         _dbusInterface->throttle();
@@ -382,6 +440,10 @@ DownloadImpl::throttle() {
 
 QString
 DownloadImpl::filePath() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return "";
+    }
     Logger::log(Logger::Debug, QString("Download{%1} filePath()").arg(_id));
     QDBusPendingReply<QString> reply =
         _dbusInterface->filePath();
@@ -399,6 +461,10 @@ DownloadImpl::filePath() {
 
 Download::State
 DownloadImpl::state() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return Download::ERROR;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} state()").arg(_id));
     QDBusPendingReply<int> reply =
         _dbusInterface->state();
@@ -421,6 +487,10 @@ DownloadImpl::id() const {
 
 qulonglong
 DownloadImpl::progress() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return 0;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} progress()").arg(_id));
     QDBusPendingReply<qulonglong> reply =
         _dbusInterface->progress();
@@ -438,6 +508,10 @@ DownloadImpl::progress() {
 
 qulonglong
 DownloadImpl::totalSize() {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return 0;
+    }
     Logger::log(Logger::Debug, QString("Download{%1} totalSize()").arg(_id));
     QDBusPendingReply<qulonglong> reply =
         _dbusInterface->totalSize();
@@ -465,21 +539,37 @@ DownloadImpl::error() const {
 
 QString
 DownloadImpl::clickPackage() const {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return "";
+    }
     return _dbusInterface->clickPackage();
 }
 
 bool
 DownloadImpl::showInIndicator() const {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return false;
+    }
     return _dbusInterface->showInIndicator();
 }
 
 QString
 DownloadImpl::title() const {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return "";
+    }
     return _dbusInterface->title();
 }
 
 QString
 DownloadImpl::destinationApp() const {
+    if (_dbusInterface == nullptr || !_dbusInterface->isValid()) {
+        Logger::log(Logger::Error, QString("Invalid dbus interface: %1").arg(_lastError->errorString()));
+        return "";
+    }
     return _dbusInterface->destinationApp();
 }
 
