@@ -32,6 +32,7 @@
 #include <QStringList>
 #include <QSslError>
 
+#include <ubuntu/transfers/i18n.h>
 #include <ubuntu/transfers/metadata.h>
 #include <ubuntu/transfers/system/dbus_connection.h>
 #include <ubuntu/transfers/system/hash_algorithm.h>
@@ -207,7 +208,7 @@ FileDownload::FileDownload(const QString& id,
     // check that the algorithm is correct if the hash is not empty
     if (!_hash.isEmpty() && !HashAlgorithm::isValidAlgo(algo)) {
         setIsValid(false);
-        setLastError(QString("Invalid hash algorithm: '%1'").arg(algo));
+        setLastError(QString(_("Invalid hash algorithm: '%1'")).arg(algo));
     }
 }
 
@@ -336,7 +337,14 @@ FileDownload::startTransfer() {
     bool canWrite = _currentData->open(QIODevice::ReadWrite | QFile::Append);
 
     if (!canWrite) {
-        emit started(false);
+        DOWN_LOG(ERROR) << "Destination file path is not writable: " << _filePath;
+        setIsValid(false);
+        if (calledFromDBus()) {
+            sendErrorReply(QDBusError::AccessDenied, QString(_("Destination file path is not writable: '%2'")).arg(
+            _filePath));
+            errorCleanup();
+            setState(Transfer::ERROR);
+        }
         return;
     }
 
@@ -860,7 +868,7 @@ FileDownload::init() {
     // ensure that the download is valid
     if (!_url.isValid()) {
         setIsValid(false);
-        setLastError(QString("Invalid URL: '%1'").arg(_url.toString()));
+        setLastError(QString(_("Invalid URL: '%1'")).arg(_url.toString()));
     }
 
     // ensure that if we are going to deflate the download that the hash is set
@@ -870,7 +878,7 @@ FileDownload::init() {
             && _metadata[Metadata::DEFLATE_KEY].toBool()) {
         setIsValid(false);
         setLastError(QString(
-            "Downloads that are set to be deflated cannot have a hash: '%1'").arg(
+            _("Downloads that are set to be deflated cannot have a hash: '%1'")).arg(
                 _hash));
     }
 }
@@ -961,7 +969,7 @@ FileDownload::initFileNames() {
         // raise an error
         if (QFile::exists(_filePath)) {
             setIsValid(false);
-            setLastError(QString("File already exists at: '%1'").arg(
+            setLastError(QString(_("File already exists at: '%2'")).arg(
                 _filePath));
         }
     } else {
@@ -1205,15 +1213,20 @@ FileDownload::buildRequest() {
     return request;
 }
 
-void
-FileDownload::emitError(const QString& error) {
-    TRACE << error;
+void 
+FileDownload::errorCleanup() {
     disconnectFromReplySignals();
     _reply->deleteLater();
     _reply = nullptr;
     cleanUpCurrentData();
     // let other downloads use the same file name
     unlockFilePath();
+}
+
+void
+FileDownload::emitError(const QString& error) {
+    TRACE << error;
+    errorCleanup();
     Download::emitError(error);
 }
 
