@@ -175,6 +175,11 @@ SingleDownload::bindDownload(Download* download)
     // after a download has finished
     m_downloadId = m_download->id();
 
+    // Allow metadata to be accessed after a download has failed or finished.
+    if (m_metadata == nullptr) {
+        setMetadata(new Metadata(download->metadata(), this));
+    }
+
     emit downloadIdChanged();
 }
 
@@ -229,9 +234,20 @@ SingleDownload::download(QString url)
                 &SingleDownload::bindDownload))
                     << "Could not connect to signal";
         }
-        Metadata metadata;
-        QMap<QString, QString> headers;
-        DownloadStruct dstruct(url, metadata.map(), headers);
+
+        QMap<QString, QString> hdrs;
+        QVariantMap _hdrs = headers();
+        foreach(const QString& key, _hdrs.keys()) {
+            hdrs[key] = _hdrs[key].toString();
+        }
+
+        QVariantMap metadataMap;
+        Metadata *meta = metadata();
+        if (meta) {
+            metadataMap = meta->map();
+        }
+
+        DownloadStruct dstruct(url, m_hash, m_algorithm, metadataMap, hdrs);
         m_manager->createDownload(dstruct);
     } else if (url.isEmpty()) {
         m_error.setMessage("No URL specified");
@@ -306,6 +322,7 @@ void
 SingleDownload::registerError(Error* error)
 {
     m_error.setMessage(error->errorString());
+    m_download = nullptr;
     emit errorFound(m_error);
     emit errorChanged();
 }
@@ -459,6 +476,16 @@ SingleDownload::metadata() const {
     }
 }
 
+QString
+SingleDownload::hash() const {
+    return m_hash;
+}
+
+QString
+SingleDownload::algorithm() const {
+    return m_algorithm;
+}
+
 void
 SingleDownload::setHeaders(QVariantMap headers) {
     if (m_download == nullptr) {
@@ -504,9 +531,9 @@ SingleDownload::setMetadata(Metadata* metadata) {
         return;
     }
 
+    m_metadata = metadata;
     if (m_download == nullptr) {
         m_dirty = true;
-        m_metadata = metadata;
     } else {
         m_download->setMetadata(metadata->map());
         if (m_download->isError()) {
@@ -522,6 +549,16 @@ SingleDownload::setMetadata(Metadata* metadata) {
     }
 }
 
+void
+SingleDownload::setHash(QString hash) {
+    m_hash = hash;
+}
+
+void
+SingleDownload::setAlgorithm(QString algorithm) {
+    m_algorithm = algorithm;
+}
+
 /*!
     \qmlproperty bool SingleDownload::autoStart
 
@@ -593,18 +630,43 @@ SingleDownload::setMetadata(Metadata* metadata) {
 /*!
     \qmlproperty Metadata SingleDownload::metadata
 
-    This property allows to get and set the metadata that will be linked to 
-    the download request. 
+    This property allows to get and set the metadata that will be linked to
+    the download request.
+*/
+
+/*!
+    \qmlproperty string SingleDownload::hash
+    \since Ubuntu.DownloadManager 1.3
+
+    This property specifies a hash to check against the downloaded file.
+    If used, this should be set prior to calling the download() method.
+*/
+
+/*!
+    \qmlproperty string SingleDownload::algorithm
+    \since Ubuntu.DownloadManager 1.3
+
+    This property indicates the algorithm to use when verifying a hash.
+    The algorithm can be of one of the following string values:
+
+         - "md5"
+         - "sha1"
+         - "sha224"
+         - "sha256"
+         - "sha384"
+         - "sha512"
+
+    If no value is specified md5 will be used.
+    If used, this should be set prior to calling the download() method.
 */
 
 /*!
     \qmlsignal SingleDownload::finished(QString path)
 
-    This signal is emitted when a download has finished. The downloaded file 
-    path is provided via the 'path' paremeter. The corresponding handler is 
+    This signal is emitted when a download has finished. The downloaded file
+    path is provided via the 'path' paremeter. The corresponding handler is
     \c onFinished
 */
 
-
 }
 }
