@@ -19,6 +19,7 @@
 #include <QTimer>
 #include <QCoreApplication>
 #include <ubuntu/uploads/daemon.h>
+#include <ubuntu/transfers/i18n.h>
 
 using namespace Ubuntu::UploadManager::Daemon;
 
@@ -29,5 +30,7 @@ int main(int argc, char *argv[]) {
     // use a singleShot timer so that we start after exec so that exit works
     QTimer::singleShot(0, daemon, SLOT(start()));
 
+    Ubuntu::Transfers::initTr(I18N_DOMAIN, NULL);
+
     return a.exec();
 }
